@@ -47,7 +47,10 @@ files = (
     'multiple_figs_demo.py',
     'pcolor_demo.py',
     'pcolor_demo2.py',
-    'pcolor_small.py',    
+    'pcolor_small.py',
+    'pie_demo.py',
+    'polar_demo.py',
+    'polar_scatter.py',
     'psd_demo.py',
     'scatter_demo.py',
     'scatter_demo2.py',
@@ -55,16 +58,29 @@ files = (
     'specgram_demo.py',
     'stock_demo.py',
     'subplot_demo.py',
+#    'set_and_get.py',    
     'table_demo.py',
     'text_handles.py',
+    'text_rotation.py',
     'text_themes.py',
     'two_scales.py',
     'vline_demo.py',
+    'zorder_demo.py',
     )
 
-def drive(backend):
 
+#tests known to fail on python22 (require datetime)
+fail22  = (
+    'date_demo1.py',
+    'date_demo2.py',    
+    'finance_demo.py',
+    )
+def drive(backend, python='python2.3'):
+    
     for fname in files:
+        if python=='python2.2' and fname in fail22:
+            print '\tSkipping %s, known to fail on python2.2'%fname
+            continue
         lines = [
             'from __future__ import division\n',
             'import matplotlib\n',
@@ -84,24 +100,25 @@ def drive(backend):
             lines.append('savefig("%s", dpi=150)' % outfile)
         tmpfile = '_tmp_%s.py' % basename
         file(tmpfile, 'w').write(''.join(lines))
-        os.system('python %s' % tmpfile)
+        os.system('%s %s' % (python, tmpfile))
 
 times = {}
-backends = ['PS', 'GD', 'Paint', 'Agg', 'Template']
-#backends.extend([ 'GTK', 'WX', 'TkAgg'])
-#backends = [ 'Agg']
-backends = [ 'SVG', 'PS', 'Agg', 'Template']
+#backends = ['Agg', 'Cairo', 'GDK', 'PS', 'SVG', 'Template']
+backends = ['Agg', 'PS', 'SVG', 'Template']
+#backends = [ 'GTK', 'WX', 'TkAgg']
 
+python = 'python2.3'
 for backend in backends:
     print 'testing %s' % backend
     t0 = time.time()
-    drive(backend)
+    drive(backend, python)
     t1 = time.time()
     times[backend] = (t1-t0)/60.0
 
 #print times
 for backend, elapsed in times.items():
     print 'Backend %s took %1.2f minutes to complete' % ( backend, elapsed)
-    print '\ttemplate ratio %1.3f, template residual %1.3f' % (
-        elapsed/times['Template'], elapsed-times['Template'])
+    if 'Template' in times:
+        print '\ttemplate ratio %1.3f, template residual %1.3f' % (
+            elapsed/times['Template'], elapsed-times['Template'])
     
