@@ -3,7 +3,7 @@
 Layer images above one another using alpha blending
 """
 from __future__ import division
-from matplotlib.matlab import *
+from pylab import *
 
 def func3(x,y):
     return (1- x/2 + x**5 + y**3)*exp(-x**2-y**2)
