@@ -1,5 +1,8 @@
 #!/usr/bin/env python
 """
+As of matplotlib-0.70, there is GUI neutral object picking.  See for
+example the picker_demo.py.
+
 Show how to use the mouse to select objects and a build dialog to set
 line properties.  The approach here can be readily extended to include
 all artists and properties.  Volunteers welcome!
@@ -12,14 +15,14 @@ import matplotlib
 matplotlib.use('GTKAgg')
 
 from matplotlib.backends.backend_gtk import NavigationToolbar, \
-     error_msg, GraphicsContextGTK
+     error_msg
 from matplotlib.backends.backend_gtkagg import FigureCanvasGTKAgg as FigureCanvas
 
 from matplotlib.figure import Figure
 from matplotlib.lines import Line2D, lineStyles, lineMarkers
 from matplotlib.transforms import Bbox, lbwh_to_bbox
 from matplotlib.patches import draw_bbox
-from matplotlib.cbook import is_string_like, True, False
+from matplotlib.cbook import is_string_like
 from matplotlib.colors import colorConverter
 
 import gtk
