@@ -7,21 +7,26 @@ from __future__ import division
 import sys
 
 from matplotlib import verbose
-from cbook import is_string_like, enumerate, True, False
+from cbook import is_string_like, enumerate, strip_math, Stack
 from colors import colorConverter
 from numerix import array, sqrt, pi, log, asarray, ones, Float
 from patches import Rectangle
 from transforms import lbwh_to_bbox
-from matplotlib import verbose
 
 class RendererBase:
+    """An abstract base class to handle drawing/rendering operations
+    """
 
     def open_group(self, s):
-        'open a grouping element with label s'
+        """open a grouping element with label s
+        Is only currently used by backend_svg
+        """
         pass
 
     def close_group(self, s):
-        'close a grouping element with label s'
+        """close a grouping element with label s
+        Is only currently used by backend_svg
+        """
         pass
     
     def get_canvas_width_height(self):
@@ -56,13 +61,12 @@ class RendererBase:
         return lbwh_to_bbox(0,0,1,1)  # your values here
 
 
-    def draw_arc(self, gcEdge, gcFace, x, y, width, height, angle1, angle2):
+    def draw_arc(self, gcEdge, rgbFace, x, y, width, height, angle1, angle2):
         """
-        Draw an arc centered at x,y with width and height and angles
-        from 0.0 to 360.0
-
-        If gcFace is not None, fill the rectangle with it.  gcEdge
-        is a GraphicsContext instance
+        Draw an arc using GraphicsContext instance gcEdge, centered at x,y,
+        with width and height and angles from 0.0 to 360.0
+        
+        If the color rgbFace is not None, fill the arc with it.
         """
         pass  # derived must override
 
@@ -202,7 +206,7 @@ class RendererBase:
         
 
     def draw_line_collection(self, segments, transform, clipbox,
-                             colors, linewidths, antialiaseds,
+                             colors, linewidths, linestyle, antialiaseds,
                              offsets, transOffset):
         """
         This is a function for optimized line drawing.  If you need to
@@ -225,6 +229,8 @@ class RendererBase:
 
         linewidths is a tuple of linewidths
 
+        linestyle is an (offset, onoffseq) tuple or None,None for solid
+
         antialiseds is a tuple of ones or zeros indicating whether the
         segment should be aa or not
 
@@ -243,6 +249,7 @@ class RendererBase:
         i = 0
         Nc = len(colors)
         Nlw = len(linewidths)
+
         Naa = len(antialiaseds)
 
         usingOffsets = offsets is not None
@@ -255,6 +262,9 @@ class RendererBase:
             
         gc.set_clip_rectangle(clipbox.get_bounds())
 
+        if linestyle[0] is not None:
+            offset, seq = linestyle
+            gc.set_dashes( offset, seq )
 
         for i in xrange(N):
             x, y = zip(*segments[i % Nsegments])
@@ -267,6 +277,7 @@ class RendererBase:
             gc.set_foreground( rgb, isRGB=True)
             gc.set_alpha( alpha )
             gc.set_linewidth( linewidths[i % Nlw] )
+    
             gc.set_antialiased( antialiaseds[i % Naa] )
             if usingOffsets:
                 xo, yo = transOffset.xy_tup(offsets[i % Noffsets])
@@ -290,33 +301,30 @@ class RendererBase:
 
     def draw_polygon(self, gc, rgbFace, points):
         """
-        Draw a polygon.  points is a len vertices tuple, each element
+        Draw a polygon using the GraphicsContext instance gc.
+        points is a len vertices tuple, each element
         giving the x,y coords a vertex
 
-        If rgbFace is not None, fill the poly with it.  gc
-        is a GraphicsContext instance
+        If the color rgbFace is not None, fill the polygon with it
         """  
         pass # derived must override
 
-    def draw_rectangle(self, gcEdge, gcFace, x, y, width, height):
+    def draw_rectangle(self, gcEdge, rgbFace, x, y, width, height):
         """
-        Draw a rectangle with lower left at x,y with width and height.
+        Draw a non-filled rectangle using the GraphicsContext instance gcEdge,
+        with lower left at x,y with width and height.
 
-        If gcFace is not None, fill the rectangle with it.  gcEdge
-        is a GraphicsContext instance
+        If rgbFace is not None, fill the rectangle with it.
         """
         pass # derived must override
 
     def strip_math(self, s):
-        remove = (r'\rm', '\cal', '\tt', '\it', '\\', '{', '}')
-        s = s[1:-1]
-        for r in remove:  s = s.replace(r,'')
-        return s
-    
+        return strip_math(s)
+              
     def draw_text(self, gc, x, y, s, prop, angle, ismath=False):
         """
-        Draw string s at x,y (display coords) with font properties
-        instance prop at angle in degrees
+        Draw the text.Text instance s at x,y (display coords) with font
+        properties instance prop at angle in degrees
         """
         pass
 
@@ -328,15 +336,16 @@ class RendererBase:
         return GraphicsContextBase()
 
 
-
-class GraphicsContextBase:    
+class GraphicsContextBase:
+    """An abstract base class that provides color, line styles, etc...
+    """
 
     # a mapping from dash styles to suggested offset, dash pairs
-    _dashd = {
+    dashd = {
         'solid'   : (None, None),
-        'dashed'  : (0, array([6.0, 6.0])),
-        'dashdot' : (0, array([3.0, 5.0, 1.0, 5.0])),
-        'dotted'  : (0, array([1.0, 3.0]))
+        'dashed'  : (0, (6.0, 6.0)),
+        'dashdot' : (0, (3.0, 5.0, 1.0, 5.0)),
+        'dotted'  : (0, (1.0, 3.0)),
               }
 
     def __init__(self):
@@ -503,19 +512,72 @@ class GraphicsContextBase:
         """
         if style in ('solid', 'dashed', 'dashdot', 'dotted'):
             self._linestyle = style
-            offset, dashes = self._dashd[style]
+            offset, dashes = self.dashd[style]
             self.set_dashes(offset, dashes)
         else:
             error_msg('Unrecognized linestyle:  Found %s' % style)
 
 
-class MplEvent:
+class Event:
     """
     A matplotlib event.  Attach additional attributes as defined in
     FigureCanvas.connect.  The following attributes are defined and
     shown with their default values
     name                # the event name
     canvas              # the FigureCanvas instance generating the event
+    
+    """
+    def __init__(self, name, canvas):
+        self.name = name
+        self.canvas = canvas
+
+class LocationEvent(Event):
+    """
+    A event that has a screen location 
+
+    The following additional attributes are defined and shown with
+    their default values
+
+    x      = None       # x position - pixels from left of canvas
+    y      = None       # y position - pixels from bottom of canvas
+    inaxes = None       # the Axes instance if mouse us over axes
+    xdata  = None       # x coord of mouse in data coords
+    ydata  = None       # y coord of mouse in data coords
+    
+    """
+    x      = None       # x position - pixels from left of canvas
+    y      = None       # y position - pixels from right of canvas
+    button = None       # button pressed None, 1, 2, 3
+    inaxes = None       # the Axes instance if mouse us over axes
+    xdata  = None       # x coord of mouse in data coords
+    ydata  = None       # y coord of mouse in data coords
+    
+    def __init__(self, name, canvas, x, y):
+        """
+        x, y in figure coords, 0,0 = bottom, left
+        button pressed None, 1, 2, 3
+        """
+        Event.__init__(self, name, canvas)
+        self.x = x
+        self.y = y
+
+        self.inaxes = None
+        for a in self.canvas.figure.get_axes():
+            if a.in_axes(self.x, self.y):
+                self.inaxes = a
+                xdata, ydata = a.transData.inverse_xy_tup((self.x, self.y))
+                self.xdata  = xdata
+                self.ydata  = ydata
+                break
+
+class MouseEvent(LocationEvent):
+    """
+    A mouse event (button_press_event, button_release_event,
+    motion_notify_event).
+
+    The following attributes are defined and shown with their default
+    values
+
     x      = None       # x position - pixels from left of canvas
     y      = None       # y position - pixels from bottom of canvas
     button = None       # button pressed None, 1, 2, 3
@@ -532,9 +594,51 @@ class MplEvent:
     xdata  = None       # x coord of mouse in data coords
     ydata  = None       # y coord of mouse in data coords
     
-    def __init__(self, name, canvas):
-        self.name = name
-        self.canvas = canvas
+    def __init__(self, name, canvas, x, y, button=None, key=None):
+        """
+        x, y in figure coords, 0,0 = bottom, left
+        button pressed None, 1, 2, 3
+        """
+        LocationEvent.__init__(self, name, canvas, x, y)
+        self.x = x
+        self.y = y
+        self.button = button
+        self.key = key
+
+        self.inaxes = None
+        for a in self.canvas.figure.get_axes():
+            if a.in_axes(self.x, self.y):
+                self.inaxes = a
+                xdata, ydata = a.transData.inverse_xy_tup((self.x, self.y))
+                self.xdata  = xdata
+                self.ydata  = ydata
+                break
+
+
+
+class KeyEvent(LocationEvent):
+    """
+    A key event (key press, key release).
+
+    Attach additional attributes as defined in
+    FigureCanvas.connect.
+
+    The following attributes are defined and shown with their default
+    values
+
+    x      = None       # x position - pixels from left of canvas
+    y      = None       # y position - pixels from bottom of canvas
+    key    = None       # the key pressed: None, chr(range(255), shift, win, or control
+    inaxes = None       # the Axes instance if mouse us over axes
+    xdata  = None       # x coord of mouse in data coords
+    ydata  = None       # y coord of mouse in data coords
+
+    This interface may change slightly when better support for
+    modifier keys is included
+    """
+    def __init__(self, name, canvas, key, x=0, y=0):
+        LocationEvent.__init__(self, name, canvas, x, y)
+        self.key = key
 
 class FigureCanvasBase:
     """
@@ -543,15 +647,72 @@ class FigureCanvasBase:
     Public attribute
 
       figure - A Figure instance
+
     """
-    events = ('button_press_event',
-              'button_release_event',
-              'motion_notify_event',
+    events = (
+        'key_press_event',
+        'key_release_event',        
+        'button_press_event',
+        'button_release_event',
+        'motion_notify_event',
               )
 
     def __init__(self, figure):
         self.figure = figure
+        self.cid = 0
+        # a dictionary from event name to a dictionary that maps cid->func
+        self.callbacks = {}
+
+        self._button     = None  # the button pressed
+        self._key        = None  # the key pressed
+        self._lastx, self.last_y = None, None
+
+    def key_press_event(self, key):
+        self._key = key
+        event = KeyEvent('key_press_event', self, key, self._lastx, self._lasty)        
+        for cid, func in self.callbacks.get('key_press_event', {}).items():
+            func(event)
+            
+    def key_release_event(self, key):
+        event = KeyEvent('key_release_event', self, key, self._lastx, self._lasty)
+        for cid, func in self.callbacks.get('key_release_event', {}).items():
+            func(event)
+        self._key = None
+
+    def button_press_event(self, x, y, button):
+        """
+        Backend derived classes should call this function on any mouse
+        button press.  x,y are the canvas coords: 0,0 is lower, left.
+        button and key are as defined in MouseEvent
+        """
+        self._button = button
+        event = MouseEvent('button_press_event', self, x, y, button, self._key)
+        for cid, func in self.callbacks.get('button_press_event', {}).items():
+            func(event)
+
+    def button_release_event(self, x, y, button):
+        """
+        Backend derived classes should call this function on any mouse
+        button release.  x,y are the canvas coords: 0,0 is lower, left.
+        button and key are as defined in MouseEvent
+        """
         
+        event = MouseEvent('button_release_event', self, x, y, button, self._key)
+        for cid, func in self.callbacks.get('button_release_event', {}).items():
+            func(event)
+        self._button = None        
+
+    def motion_notify_event(self, x, y):
+        """
+        Backend derived classes should call this function on any mouse
+        button release.  x,y are the canvas coords: 0,0 is lower, left.
+        button and key are as defined in MouseEvent
+        """
+        self._lastx, self._lasty = x, y
+        event = MouseEvent('motion_notify_event', self, x, y, self._button, self._key)
+        for cid, func in self.callbacks.get('motion_notify_event', {}).items():
+            func(event)
+
     def draw(self):
         """
         Render the figure
@@ -566,6 +727,7 @@ class FigureCanvasBase:
         face color background and you'll probably want to override this on
         hardcopy.
 
+        filename    - can also be a file object on image backends
         orientation - only currently applies to PostScript printing.
         """
 
@@ -589,6 +751,7 @@ class FigureCanvasBase:
 
         where event is a MplEvent.  The following events are recognized
 
+         'key_press_event' 
          'button_press_event' 
          'button_release_event' 
          'motion_notify_event' 
@@ -601,14 +764,20 @@ class FigureCanvasBase:
 
         return value is a connection id that can be used with
         mpl_disconnect """
-        pass
+        self.cid += 1
+        self.callbacks.setdefault(s, {})[self.cid] = func
+        return self.cid
         
     def mpl_disconnect(self, cid):
         """
         Connect s to func. return an id that can be used with disconnect
         Method should return None
         """
-        return None
+        for eventname, callbackd in self.callbacks.items():
+            if callbackd.has_key(cid):
+                del callbackd[cid]
+                return
+
 
 class FigureManagerBase:
     """
@@ -621,123 +790,22 @@ class FigureManagerBase:
     def __init__(self, canvas, num):
         self.canvas = canvas
         self.num = num
-        self.clf()
-        
+
     def clf(self):
         'clear the figure'
-        self.axes = {}
-        self.currentAxis = None
+        verbose.report_error('Deprectated; use fig.clf() instead')
         self.canvas.figure.clf()
         
-    def add_subplot(self, *args, **kwargs):
-        """
-        Add a subplot to the current figure
-        """
-        key = (args, tuple(kwargs.items()))
-        if self.axes.has_key(key):
-            self.currentAxis = self.axes[key]
-        else:
-            a = self.canvas.figure.add_subplot(*args, **kwargs)
-            self.axes[key] = a
-            self.currentAxis = a
-            return a
-        
-    def add_axes(self, rect, **kwargs):
-        """
-        Add an axes to the current figure
-        """
-        rect = tuple(rect)
-        key = (rect, tuple(kwargs.items()))
-        if self.axes.has_key(key):
-            self.currentAxis = self.axes[key]
-            return self.currentAxis
-        else:
-            a = self.canvas.figure.add_axes(rect, **kwargs)
-            self.axes[key] = a
-            self.currentAxis = a
-            return a
-
-    def get_current_axis(self, **kwargs):
-        """
-        Return the current axes
-        """
-        if self.currentAxis is not None:
-            return self.currentAxis
-        else:
-            self.add_subplot(111, **kwargs)
-            return self.currentAxis
-
-
-    def set_current_axes(self, a):
-        """
-        Set the current axes to be a
-        """
-        if a is None:
-            self.currentAxis = None
-            return
-        if a not in self.axes.values():
-            error_msg('Axes is not in current figure')
-        self.currentAxis = a
 
     def destroy(self):
         pass
 
-
 # cursors
 class Cursors:  #namespace
     HAND, POINTER, SELECT_REGION, MOVE = range(4)
 cursors = Cursors()
 
 
-class Stack:
-    """
-    Implement a stack where elements can be pushed on and you can move
-    back and forth.  But no pop.  Should mimib home / back / forward
-    in a browser
-    """
-
-    def __init__(self):
-        self.clear()
-        
-    def __call__(self):
-        'return the current element, or None'
-        if not len(self._elements): return None
-        else: return self._elements[self._pos]
-
-    def forward(self):
-        'move the position forward and return the current element'
-        N = len(self._elements)
-        if self._pos<N-1: self._pos += 1
-        return self()
-
-    def back(self):
-        'move the position back and return the current element'
-        if self._pos>0: self._pos -= 1
-        return self()
-
-    def push(self, o):
-        """
-        push object onto stack at current position - all elements
-        occurring later than the current position are discarded
-        """
-        self._elements = self._elements[:self._pos+1]
-        self._elements.append(o)
-        self._pos = len(self._elements)-1
-        return self()
-    
-    def home(self):
-        'push the first element onto the top of the stack'
-        if not len(self._elements): return
-        self.push(self._elements[0])
-        return self()
-
-    def empty(self):
-        return len(self._elements)==0
-
-    def clear(self):
-        'empty the stack'
-        self._pos = -1
-        self._elements = []
 
 class NavigationToolbar2:
     """
@@ -1070,7 +1138,7 @@ class NavigationToolbar2:
         
         lastx, lasty, a, ind, lim, trans = self._xypress
         # ignore singular clicks - 5 pixels is a threshold
-        if abs(x-lastx)<5 or abs(y-lasty)<5 or not a.in_axes(x,y):
+        if abs(x-lastx)<5 or abs(y-lasty)<5:
             self._xypress = None
             self.release(event)
             self.draw()
@@ -1080,20 +1148,36 @@ class NavigationToolbar2:
 
         # zoom to rect
         lastx, lasty = a.transData.inverse_xy_tup( (lastx, lasty) )
-        x, y = a.transData.inverse_xy_tup( (x, y) )            
+        x, y = a.transData.inverse_xy_tup( (x, y) )  
+        Xmin,Xmax=a.get_xlim()
+        Ymin,Ymax=a.get_ylim()          
 
-        if x<lastx:  xmin, xmax = x, lastx
-        else: xmin, xmax = lastx, x
-
-        if y<lasty:  ymin, ymax = y, lasty
-        else: ymin, ymax = lasty, y
+        if Xmin < Xmax:
+            if x<lastx:  xmin, xmax = x, lastx
+            else: xmin, xmax = lastx, x  
+            if xmin < Xmin: xmin=Xmin
+            if xmax > Xmax: xmax=Xmax
+        else: 
+            if x>lastx:  xmin, xmax = x, lastx
+            else: xmin, xmax = lastx, x  
+            if xmin > Xmin: xmin=Xmin
+            if xmax < Xmax: xmax=Xmax
+                
+        if Ymin < Ymax:
+            if y<lasty:  ymin, ymax = y, lasty
+            else: ymin, ymax = lasty, y  
+            if ymin < Ymin: ymin=Ymin
+            if ymax > Ymax: ymax=Ymax
+        else: 
+            if y>lasty:  ymin, ymax = y, lasty
+            else: ymin, ymax = lasty, y  
+            if ymin > Ymin: ymin=Ymin
+            if ymax < Ymax: ymax=Ymax
 
         if self._button_pressed == 1:  
             a.set_xlim((xmin, xmax))
             a.set_ylim((ymin, ymax))
         elif self._button_pressed == 3: 
-            Xmin,Xmax=a.get_xlim()
-            Ymin,Ymax=a.get_ylim()
             if a.get_xscale()=='log':
                 alpha=log(Xmax/Xmin)/log(xmax/xmin)
                 x1=pow(Xmin/xmin,alpha)*Xmin
