@@ -92,7 +92,7 @@ Examples which work on this release:
  (3) - Clipping seems to be broken.
 """
 
-cvs_id = '$Id: backend_wx.py,v 1.2 2004/09/30 19:33:06 jdh2358 Exp $'
+cvs_id = '$Id: backend_wx.py,v 1.13 2004/12/30 17:10:15 jdh2358 Exp $'
 
 import sys, os, os.path, math, StringIO
 
@@ -152,8 +152,8 @@ import matplotlib
 from matplotlib import verbose
 from matplotlib.backend_bases import RendererBase, GraphicsContextBase,\
      FigureCanvasBase, FigureManagerBase, error_msg, NavigationToolbar2, \
-     MplEvent, cursors
-from matplotlib._matlab_helpers import Gcf
+     cursors
+from matplotlib._pylab_helpers import Gcf
 from matplotlib.artist import Artist
 from matplotlib.cbook import exception_to_str
 from matplotlib.figure import Figure
@@ -691,9 +691,6 @@ class FigureCanvasWx(FigureCanvasBase, wxPanel):
         l,b,w,h = figure.bbox.get_bounds()
         w = int(math.ceil(w))
         h = int(math.ceil(h))
-
-        self._key = None
-
         
         wxPanel.__init__(self, parent, id, size=wxSize(w, h))
         # Create the drawing bitmap
@@ -713,64 +710,128 @@ class FigureCanvasWx(FigureCanvasBase, wxPanel):
         EVT_MOUSEWHEEL(self, self._onMouseWheel)
         EVT_LEFT_DOWN(self, self._onLeftButtonDown)
         EVT_LEFT_UP(self, self._onLeftButtonUp)
-
+        EVT_MOTION(self, self._onMotion)
         self.macros = {} # dict from wx id to seq of macros
-
         
-    def mpl_disconnect(self, cid):
-        for macro in self.macros.get(cid, []):
-            macro(self, None)
-        return None
+        self.Printer_Init()
 
+    def Copy_to_Clipboard(self, event=None):
+        "copy bitmap of canvas to system clipboard"
+        bmp_obj = wx.BitmapDataObject()
+        bmp_obj.SetBitmap(self.bitmap)
+        wx.TheClipboard.Open()
+        wx.TheClipboard.SetData(bmp_obj)
+        wx.TheClipboard.Close()
+
+    def Printer_Init(self):
+        """initialize printer settings using wx methods"""
+        self.printerData = wx.PrintData()
+        self.printerData.SetPaperId(wx.PAPER_LETTER)
+        self.printerData.SetPrintMode(wx.PRINT_MODE_PRINTER)
+        self.printerPageData= wx.PageSetupDialogData()
+        self.printerPageData.SetMarginBottomRight((25,25))
+        self.printerPageData.SetMarginTopLeft((25,25))
+        self.printerPageData.SetPrintData(self.printerData)
+
+        self.printer_width = 5.5
+        self.printer_margin= 0.5
         
-    def mpl_connect(self, s, func):
+    def Printer_Setup(self, event=None):
+        """set up figure for printing.  The standard wx Printer
+        Setup Dialog seems to die easily. Therefore, this setup
+        simply asks for image width and margin for printing. """
+        
+        dmsg = """Width of output figure in inches.
+The current aspect ration will be kept."""
 
-        if s not in self.events:
-            error_msg('Can only connect events of type "%s"\nDo not know how to handle "%s"' %(', '.join(self.events), s))    
-            
-        cid = wxNewId()
+        dlg = wx.Dialog(self, -1, 'Page Setup for Printing' , (-1,-1))
+        df = dlg.GetFont()
+        df.SetWeight(wx.NORMAL)
+        df.SetPointSize(11)
+        dlg.SetFont(df)
 
-        def wrapper(event):
-            thisEvent = MplEvent(s, self) 
+        x_wid = wx.TextCtrl(dlg,-1,value="%.2f" % self.printer_width, size=(70,-1))
+        x_mrg = wx.TextCtrl(dlg,-1,value="%.2f" % self.printer_margin,size=(70,-1))
+        
+        sizer = wx.GridBagSizer(3,3)
+        sizer.Add(wx.StaticText(dlg,-1,dmsg),
+                  (0,0),(1,3), wx.ALIGN_LEFT|wx.ALL|wx.EXPAND, 5)
 
-            thisEvent.x = event.GetX()
-            # flipy so y=0 is bottom of canvas
-            thisEvent.y = self.figure.bbox.height() - event.GetY()
+        sizer.Add(wx.StaticText(dlg,-1,'Figure Width'), 
+                  (1,0),(1,1), wx.ALIGN_LEFT|wx.ALL, 2)
+        sizer.Add(x_wid,
+                  (1,1),(1,1), wx.ALIGN_LEFT|wx.ALL, 2)
+        sizer.Add(wx.StaticText(dlg,-1,'in'), 
+                  (1,2),(1,1), wx.ALIGN_LEFT|wx.ALL, 2)
 
-            if event.LeftDown(): button = 1
-            elif event.MiddleDown(): button = 2
-            elif event.RightDown(): button = 3
-            else: button = None
-            thisEvent.button = button
-            thisEvent.key = self._key
-
-            thisEvent.inaxes = None
-            for a in self.figure.get_axes():
-                if a.in_axes(thisEvent.x, thisEvent.y):
-                    thisEvent.inaxes = a
-                    xdata, ydata = a.transData.inverse_xy_tup((thisEvent.x, thisEvent.y))
-                    thisEvent.xdata  = xdata
-                    thisEvent.ydata  = ydata
-                    break
-                
-            
-            func(thisEvent)
-            event.Skip()
-            return False  # return True blocks other connects
+        sizer.Add(wx.StaticText(dlg,-1,'Margin'), 
+                  (2,0),(1,1), wx.ALIGN_LEFT|wx.ALL, 2)
+        sizer.Add(x_mrg,
+                  (2,1),(1,1), wx.ALIGN_LEFT|wx.ALL, 2)
+        sizer.Add(wx.StaticText(dlg,-1,'in'), 
+                  (2,2),(1,1), wx.ALIGN_LEFT|wx.ALL, 2)
 
 
-        if s=='button_press_event':
-            EVT_LEFT_DOWN(self,  wrapper)
-            EVT_RIGHT_DOWN(self, wrapper)
-            self.macros[cid] = (EVT_LEFT_DOWN, EVT_RIGHT_DOWN)
-        if s=='button_release_event':
-            EVT_LEFT_UP(self,  wrapper)            
-            EVT_RIGHT_UP(self, wrapper)
-            self.macros[cid] = (EVT_LEFT_UP, EVT_RIGHT_UP)
-        elif s=='motion_notify_event':
-            EVT_MOTION(self, wrapper)
-            self.macros[cid] = (EVT_MOTION,)
-        return cid
+        btn = wx.Button(dlg,wx.ID_OK, " OK ")
+        btn.SetDefault()
+        sizer.Add(btn, (3,0),(1,1), wx.ALIGN_LEFT, 5)
+        btn = wx.Button(dlg,wx.ID_CANCEL, " CANCEL ")
+        sizer.Add(btn, (3,1),(1,1), wx.ALIGN_LEFT, 5)
+        
+        dlg.SetSizer(sizer)
+        dlg.SetAutoLayout(True)
+        sizer.Fit(dlg)
+        
+        if dlg.ShowModal() == wx.ID_OK:
+            try:
+                self.printer_width  = float(x_wid.GetValue())
+                self.printer_margin = float(x_mrg.GetValue())
+            except:
+                pass
+
+        if ((self.printer_width + self.printer_margin) > 7.5):
+            self.printerData.SetOrientation(wx.LANDSCAPE)
+        else:
+            self.printerData.SetOrientation(wx.PORTRAIT)
+        dlg.Destroy()
+        return
+
+    def Printer_Preview(self, event=None):
+        """ generate Print Preview with wx Print mechanism"""
+        po1  = PrintoutWx(self, width=self.printer_width,
+                          margin=self.printer_margin)
+        po2  = PrintoutWx(self, width=self.printer_width,
+                          margin=self.printer_margin)
+        self.preview = wx.PrintPreview(po1,po2,self.printerData)
+        if not self.preview.Ok():  print "error with preview"
+
+        self.preview.SetZoom(50)
+        frameInst= self
+        while not isinstance(frameInst, wx.Frame):
+            frameInst= frameInst.GetParent()
+        frame = wx.PreviewFrame(self.preview, frameInst, "Preview")
+        frame.Initialize()
+        frame.SetPosition(self.GetPosition())
+        frame.SetSize((850,650))  
+        frame.Centre(wx.BOTH)        
+        frame.Show(True)
+        self.gui_repaint()
+
+    def Printer_Print(self, event=None):
+        """ Print figure using wx Print mechanism"""
+        pdd = wx.PrintDialogData(self.printerData)
+        pdd.SetToPage(1)
+        printer  = wx.Printer(pdd)
+        printout  = PrintoutWx(self, width=self.printer_width,
+                               margin=self.printer_margin)
+        print_ok = printer.Print(self, printout, True)
+        if not print_ok:
+            wx.MessageBox("""There was a problem printing.
+            Perhaps your current printer is not set correctly?""",
+                          "Printing", wx.OK)
+        printout.Destroy()
+        self.gui_repaint()
+
                
     def draw(self):
         """
@@ -974,10 +1035,8 @@ class FigureCanvasWx(FigureCanvasBase, wxPanel):
             self.draw()
         evt.Skip()
         
+    def _get_key(self, evt):
 
-    def _onKeyDown(self, evt):
-        """Capture key press."""
-        evt.Skip()
         keyval = evt.m_keyCode
         if self.keyvald.has_key(keyval):
             key = self.keyvald[keyval]
@@ -985,45 +1044,72 @@ class FigureCanvasWx(FigureCanvasBase, wxPanel):
             key = chr(keyval)
         else:
             key = None
+ 
+        # why is wx upcasing this?
+        if key is not None: key = key.lower()
             
-        if key: self._key = key.lower()
-        else:   self._key = key
-        
+        return key
+
+    def _onKeyDown(self, evt):
+        """Capture key press."""
+        key = self._get_key(evt)
+        evt.Skip()
+        FigureCanvasBase.key_press_event(self, key)
+    
     def _onKeyUp(self, evt):
         """Release key."""
+        key = self._get_key(evt)
+        #print 'release key', key
         evt.Skip()
-        self._key = None
-
+        FigureCanvasBase.key_release_event(self, key)
+ 
     def _onRightButtonDown(self, evt):
         """Start measuring on an axis."""
-        #print 'left down', evt, dir(evt)
+        x = evt.GetX()
+        y = self.figure.bbox.height() - evt.GetY()
         evt.Skip()
-        
+        FigureCanvasBase.button_press_event(self, x, y, 3)        
+
+
     def _onRightButtonUp(self, evt):
         """End measuring on an axis."""
+        x = evt.GetX()
+        y = self.figure.bbox.height() - evt.GetY()
         evt.Skip()
-
+        FigureCanvasBase.button_release_event(self, x, y, 3)        
 
     def _onLeftButtonDown(self, evt):
         """Start measuring on an axis."""
-
+        x = evt.GetX()
+        y = self.figure.bbox.height() - evt.GetY()
         evt.Skip()
-        
+        FigureCanvasBase.button_press_event(self, x, y, 1)        
+
     def _onLeftButtonUp(self, evt):
         """End measuring on an axis."""
+        x = evt.GetX()
+        y = self.figure.bbox.height() - evt.GetY()
+        #print 'release button', 1
         evt.Skip()
-        
+        FigureCanvasBase.button_release_event(self, x, y, 1)
+
     def _onMouseWheel(self, evt):
         # TODO: implement mouse wheel handler
         pass
 
+    def _onMotion(self, evt):
+        """Start measuring on an axis."""
+        
+        x = evt.GetX()
+        y = self.figure.bbox.height() - evt.GetY()
+        evt.Skip()
+        FigureCanvasBase.motion_notify_event(self, x, y)
 
-    
 
 ########################################################################
 #    
-# The following functions and classes are for matlab compatibility
-# mode (matplotlib.matlab) and implement figure managers, etc...
+# The following functions and classes are for pylab compatibility
+# mode (matplotlib.pylab) and implement figure managers, etc...
 #
 ########################################################################
 
@@ -1072,7 +1158,7 @@ def new_figure_manager(num, *args, **kwargs):
     """
     Create a new figure manager instance
     """
-    # in order to expose the Figure constructor to the matlab
+    # in order to expose the Figure constructor to the pylab
     # interface we need to create the figure here
     DEBUG_MSG("new_figure_manager()", 3, None)
     fig = Figure(*args, **kwargs)
@@ -1098,7 +1184,8 @@ class FigureFrameWx(wxFrame):
         self.num = num
 
         self.canvas = self.get_canvas(fig)
-        self.SetStatusBar(StatusBarWx(self))
+        statbar = StatusBarWx(self)
+        self.SetStatusBar(statbar)
         self.sizer = wxBoxSizer(wxVERTICAL)
         self.sizer.Add(self.canvas, 1, wxTOP | wxLEFT | wxEXPAND)
         # By adding toolbar in sizer, we are able to put it at the bottom
@@ -1112,7 +1199,7 @@ class FigureFrameWx(wxFrame):
         else:
             self.toolbar = None
         
-
+        self.toolbar.set_status_bar(statbar)
         if self.toolbar is not None:
             self.toolbar.Realize()
             if wxPlatform == '__WXMAC__':
@@ -1161,7 +1248,12 @@ class FigureManagerWx(FigureManagerBase):
     It is instantiated by GcfWx whenever a new figure is created. GcfWx is
     responsible for managing multiple instances of FigureManagerWx.
     
-    NB: FigureManagerBase is found in _matlab_helpers
+    NB: FigureManagerBase is found in _pylab_helpers
+
+    public attrs
+
+    canvas - a FigureCanvasWx(wxPanel) instance
+    window - a wxFrame instance - http://www.lpthe.jussieu.fr/~zeitlin/wxWindows/docs/wxwin_wxframe.html#wxframe
     """
     def __init__(self, canvas, num, frame):
         DEBUG_MSG("__init__()", 1, self)
@@ -1170,23 +1262,10 @@ class FigureManagerWx(FigureManagerBase):
         self.window = frame
         self.tb = frame.GetToolBar()
 
-    def add_subplot(self, *args, **kwargs):
-        DEBUG_MSG("add_subplot()", 1, self)
-        a = FigureManagerBase.add_subplot(self, *args, **kwargs)
-        if self.tb is not None: self.tb.update()
-        return a
-        
-    def add_axes(self, rect, **kwargs):
-        DEBUG_MSG("add_axes()", 1, self)
-        a = FigureManagerBase.add_axes(self, rect, **kwargs)
-        if self.tb is not None: self.tb.update()
-        return a
-    
-    def set_current_axes(self, a):
-        DEBUG_MSG("set_current_axes()", 1, self)
-        if a not in self.axes.values():
-            error_msg_wx('Axes is not in current figure')
-        FigureManagerBase.set_current_axes(self, a)
+        def notify_axes_change(fig):
+            'this will be called whenever the current axes is changed'        
+            if self.tb != None: self.tb.update()
+        self.canvas.figure.add_axobserver(notify_axes_change)
         
     def destroy(self, *args):
         DEBUG_MSG("destroy()", 1, self)
@@ -1344,7 +1423,7 @@ class NavigationToolbar2Wx(NavigationToolbar2, wxToolBar):
         NavigationToolbar2.__init__(self, canvas)
         self.canvas = canvas
         self._idle = True
-        
+        self.statbar = None        
     def _init_toolbar(self):
         DEBUG_MSG("_init_toolbar", 1, self)
 
@@ -1453,7 +1532,12 @@ class NavigationToolbar2Wx(NavigationToolbar2, wxToolBar):
         self.lastrect = rect
         dc.DrawRectangle(*rect)
         dc.EndDrawing()
-               
+
+    def set_status_bar(self, statbar):
+        self.statbar = statbar
+
+    def set_message(self, s):
+        if self.statbar is not None: self.statbar.set_function(s)
 
 class NavigationToolbarWx(wxToolBar):
     def __init__(self, canvas, can_kill=False):
@@ -1677,16 +1761,98 @@ class StatusBarWx(wxStatusBar):
     """
     def __init__(self, parent):
         wxStatusBar.__init__(self, parent, -1)
-        self.SetFieldsCount(3)
-        self.SetStatusText("Function: None", 1)
-        self.SetStatusText("Measurement: None", 2)
+        self.SetFieldsCount(2)
+        self.SetStatusText("None", 1)
+        #self.SetStatusText("Measurement: None", 2)
         #self.Reposition()
         
     def set_function(self, string):
-        self.SetStatusText("Function: %s" % string, 1)
+        self.SetStatusText("%s" % string, 1)
         
-    def set_measurement(self, string):
-        self.SetStatusText("Measurement: %s" % string, 2)
+    #def set_measurement(self, string):
+    #    self.SetStatusText("Measurement: %s" % string, 2)
+
+
+#< Additions for printing support: Matt Newville 
+
+class PrintoutWx(wx.Printout):
+    """Simple wrapper around wx Printout class -- all the real work
+    here is scaling the matplotlib canvas bitmap to the current
+    printer's definition.
+    """
+    def __init__(self, canvas, width=5.5,margin=0.5, title='matplotlib'):
+        wx.Printout.__init__(self,title=title)
+        self.canvas = canvas
+        # width, in inches of output figure (approximate)
+        self.width  = width
+        self.margin = margin
+        
+    def HasPage(self, page):
+        #current only supports 1 page print
+        return page == 1
+    
+    def GetPageInfo(self):
+        return (1, 1, 1, 1)
+
+    def OnPrintPage(self, page):
+        self.canvas.draw()
+        
+        dc        = self.GetDC()
+        (ppw,pph) = self.GetPPIPrinter()      # printer's pixels per in
+        (pgw,pgh) = self.GetPageSizePixels()  # page size in pixels
+        (dcw,dch) = dc.GetSize()
+        (grw,grh) = self.canvas.GetSizeTuple()
+        
+        # save current figure dpi resolution and bg color,
+        # so that we can temporarily set them to the dpi of
+        # the printer, and the bg color to white
+        bgcolor   = self.canvas.figure.get_facecolor()        
+        fig_dpi   = self.canvas.figure.dpi.get()
+
+        # draw the bitmap, scaled appropriately
+        vscale    = ppw / fig_dpi
+
+        # set figure resolution,bg color for printer
+        self.canvas.figure.dpi.set(ppw)
+        self.canvas.figure.set_facecolor('#FFFFFF')
+
+
+        renderer  = RendererWx(self.canvas.bitmap, self.canvas.figure.dpi)
+        self.canvas.figure.draw(renderer)
+        self.canvas.bitmap.SetWidth( self.canvas.bitmap.GetWidth() * vscale)
+        self.canvas.bitmap.SetHeight(self.canvas.bitmap.GetHeight()* vscale)
+        self.canvas.draw()
+
+        # page may need additional scaling on preview
+        page_scale = 1.0
+        if self.IsPreview():   page_scale = float(dcw)/pgw
+
+        # get margin in pixels = (margin in in) * (pixels/in)
+        top_margin  = int(self.margin * pph * page_scale)
+        left_margin = int(self.margin * ppw * page_scale)
+        
+        # set scale so that width of output is self.width inches
+        # (assuming grw is size of graph in inches....)
+        user_scale = (self.width * fig_dpi * page_scale)/float(grw)
+
+        dc.SetDeviceOrigin(left_margin,top_margin)
+        dc.SetUserScale(user_scale,user_scale)
+
+        # this cute little number avoid API inconsistencies in wx
+        try:
+            dc.DrawBitmap(self.canvas.bitmap, 0, 0)
+        except:
+            try:
+                dc.DrawBitmap(self.canvas.bitmap, (0, 0))
+            except:
+                pass
+            
+        # restore original figure  resolution
+        self.canvas.figure.set_facecolor(bgcolor)
+        self.canvas.figure.dpi.set(fig_dpi)
+        self.canvas.draw()
+        return True
+#>
 
 ########################################################################
 #    
