@@ -1,9 +1,9 @@
 """
 
-Numerical python functions written for compatability with matlab
+Numerical python functions written for compatability with matlab(TM)
 commands with the same names.  
 
-  Matlab compatible functions:
+  Matlab(TM) compatible functions:
 
     * cohere - Coherence (normalized cross spectral density)
 
@@ -37,7 +37,7 @@ commands with the same names.
 
     * trapz - trapeziodal integration
     
-  Functions that don't exist in matlab, but are useful anyway:
+  Functions that don't exist in matlab(TM), but are useful anyway:
 
     * cohere_pairs - Coherence over all pairs.  This is not a matlab
       function, but we compute coherence a lot in my lab, and we
@@ -55,12 +55,12 @@ Credits:
 """
 
 from __future__ import division
-import sys
+import sys, random
 from matplotlib import verbose
 import numerix
 from numerix import *
 
-from cbook import iterable, True, False
+from cbook import iterable
 
 
 def mean(x, dim=None):
@@ -634,7 +634,7 @@ def levypdf(x, gamma, alpha):
 
    dx = x[1]-x[0]
 
-   Nyq = 1/(2*dx)
+
    f = 1/(N*dx)*arange(-N/2, N/2, Float)
 
    ind = concatenate([arange(N/2, N, Int),
@@ -679,8 +679,8 @@ def longest_contiguous_ones(x):
     y = zeros( (len(x)+2,),  x.typecode())
     y[1:-1] = x
     dif = diff(y)
-    up = find(d ==  1);
-    dn = find(d == -1);
+    up = find(dif ==  1);
+    dn = find(dif == -1);
     ind = find( dn-up == max(dn - up))
     ind = arange(take(up, ind), take(dn, ind))
 
@@ -1141,6 +1141,40 @@ def bivariate_normal(X, Y, sigmax=1.0, sigmay=1.0,
 
 
 
+def get_xyz_where(Z, Cond):
+    """
+    Z and Cond are MxN matrices.  Z are data and Cond is a boolean
+    matrix where some condition is satisfied.  Return value is x,y,z
+    where x and y are the indices into Z and z are the values of Z at
+    those indices.  x,y,z are 1D arrays
+    """
+    
+    M,N = Z.shape
+    z = ravel(Z)
+    ind = nonzero( ravel(Cond) )
+
+    x = arange(M); x.shape = M,1
+    X = repeat(x, N, 1)
+    x = ravel(X)
+
+    y = arange(N); y.shape = 1,N
+    Y = repeat(y, M)
+    y = ravel(Y)
+
+    x = take(x, ind)
+    y = take(y, ind)
+    z = take(z, ind)
+    return x,y,z
+
+def get_sparse_matrix(M,N,frac=0.1):
+    'return a MxN sparse matrix with frac elements randomly filled'
+    data = zeros((M,N))*0.
+    for i in range(int(M*N*frac)):
+        x = random.randint(0,M-1)
+        y = random.randint(0,N-1)
+        data[x,y] = rand()
+    return data
+
 
 ### the following code was written and submitted by Fernando Perez
 ### from the ipython numutils package under a BSD license
