@@ -6,8 +6,7 @@ This module contains classes to support completely configurable tick
 locating and formatting.  Although the locators know nothing about
 major or minor ticks, they are used by the Axis class to support major
 and minor tick locating and formatting.  Generic tick locators and
-formatters are provided, as well as domain specific custom locators an
-formatters.
+formatters are provided, as well as domain specific custom ones..
 
 
 Tick locating
@@ -15,7 +14,7 @@ Tick locating
 
 The Locator class is the base class for all tick locators.  The
 locators handle autoscaling of the view limits based on the data
-limits, and choosing the tick locations.  The most generally useful
+limits, and the choosing of tick locations.  The most generally useful
 tick locator is MultipleLocator.  You initialize this with a base, eg
 10, and it picks axis limits and ticks that are multiples of your
 base.  The class AutoLocator contains a MultipleLocator instance, and
@@ -111,6 +110,7 @@ multiple axes can share the same locator w/o side effects!
 from __future__ import division
 import sys, os, re, time, math
 from mlab import linspace
+from matplotlib import verbose
 from numerix import arange, array, asarray, ones, zeros, \
      nonzero, take, Float, log, logical_and
 
@@ -234,6 +234,8 @@ class ScalarFormatter(Formatter):
 class LogFormatter(ScalarFormatter):
     """
     Format values for log axis; 
+
+    if attribute decadeOnly is True, only the decades will be labelled.
     """
     def __init__(self, base=10.0, labelOnlyBase = True):
         """
@@ -243,7 +245,8 @@ class LogFormatter(ScalarFormatter):
         """
         self.base = base+0.0
         self.labelOnlyBase=labelOnlyBase
-    
+        self.decadeOnly = True
+
     def base(self,base):
         'change the base for labeling - warning: should always match the base used for LogLocator' 
         self.base=base
@@ -251,6 +254,7 @@ class LogFormatter(ScalarFormatter):
     def label_minor(self,labelOnlyBase):
         'switch on/off minor ticks labeling' 
         self.labelOnlyBase=labelOnlyBase
+
         
     def __call__(self, x, pos=0):
         'Return the format for tick val x at position pos'        
@@ -306,7 +310,7 @@ class LogFormatterMathtext(LogFormatter):
     def __call__(self, x, pos=0):
         'Return the format for tick val x at position pos'        
         self.verify_intervals()
-        d = abs(self.viewInterval.span())
+
         b = self.base
         # only label the decades
         fx = math.log(x)/math.log(b)
@@ -316,7 +320,7 @@ class LogFormatterMathtext(LogFormatter):
         if not isDecade and self.labelOnlyBase: s = ''
         elif not isDecade: s = '$%d^{%.2f}$'% (b, fx)
         else: s = '$%d^{%d}$'% (b, self.nearest_long(fx))
-        #print x, b, fx, isDecade, s
+
         return s
 
 
@@ -448,7 +452,7 @@ class LinearLocator(Locator):
 
         if self.numticks==0: return []
         ticklocs = linspace(vmin, vmax, self.numticks)
-        #print 'linear', ticklocs
+
         return ticklocs
 
 
@@ -466,11 +470,9 @@ class LinearLocator(Locator):
         if vmin==vmax:
             vmin-=1
             vmax+=1
-        try:
-            (exponent, remainder) = divmod(math.log10(vmax - vmin), 1)
-        except OverflowError:
-            print >>sys.stderr, 'Overflow error in autoscale', vmin, vmax
-            return
+
+        exponent, remainder = divmod(math.log10(vmax - vmin), 1)
+
         if remainder < 0.5:
             exponent -= 1
         scale = 10**(-exponent)
@@ -555,17 +557,14 @@ class MultipleLocator(Locator):
 
         self.verify_intervals()
         dmin, dmax = self.dataInterval.get_bounds()
-        #print 'data limits', dmin, dmax, (dmax-dmin)/60.0
 
         vmin = self.base.le(dmin)
         vmax = self.base.ge(dmax)
-        #print 'base rounded', vmin, vmax, (vmax-vmin)/60.0
         if vmin==vmax:
             vmin -=1
             vmax +=1
 
         
-        #print 'mult locator returning', vmin, vmax, (vmax-vmin)/60.0
         return self.nonsingular(vmin, vmax)
 
 def decade_down(x, base=10):
@@ -689,7 +688,7 @@ class AutoLocator(Locator):
 
             try: ld = math.log10(d)
             except OverflowError:
-                print >> sys.stderr, 'AutoLocator illegal dataInterval range %s; returning NullLocator'%d
+                verbose.report_error('AutoLocator illegal dataInterval range %s; returning NullLocator'%d)
                 return NullLocator()
 
             fld = math.floor(ld)
