@@ -13,17 +13,20 @@
 
 #include "agg_pixfmt_rgb24.h"
 #include "agg_pixfmt_rgba32.h"
+#include "agg_pixfmt_rgba32.h"
+#include "agg_color_rgba8.h"
 #include "agg_rendering_buffer.h"
 #include "agg_rasterizer_scanline_aa.h"
+#include "agg_scanline_bin.h"
 #include "agg_path_storage.h"
 #include "agg_conv_transform.h"
 #include "agg_span_image_filter_rgb24.h"
 #include "agg_span_image_filter_rgba32.h"
 #include "agg_span_interpolator_linear.h"
-#include "agg_scanline_u8.h"
-#include "agg_scanline_p32.h"
+#include "agg_scanline_bin.h"
+#include "agg_scanline_u.h"
 #include "agg_renderer_scanline.h"
-
+#include "util/agg_color_conv_rgb8.h"
 #include "_image.h"
 #include "mplutils.h"
 
@@ -33,7 +36,7 @@ typedef agg::pixel_formats_rgba32<agg::order_rgba32> pixfmt;
 typedef agg::renderer_base<pixfmt> renderer_base;
 typedef agg::span_interpolator_linear<> interpolator_type;
 typedef agg::span_image_filter_rgba32_bilinear<agg::order_rgba32, interpolator_type> span_gen_type;
-typedef agg::renderer_scanline_u<renderer_base, span_gen_type> renderer_type;
+typedef agg::renderer_scanline_aa<renderer_base, span_gen_type> renderer_type;
 typedef agg::rasterizer_scanline_aa<> rasterizer;
 
 
@@ -82,7 +85,7 @@ Image::apply_rotation(const Py::Tuple& args) {
   double r = Py::Float(args[0]);
   
   
-  agg::affine_matrix M = agg::rotation_matrix( r * agg::pi / 180.0);	      
+  agg::trans_affine M = agg::trans_affine_rotation( r * agg::pi / 180.0);	      
   srcMatrix *= M;
   imageMatrix *= M;
   return Py::Object();  
@@ -122,7 +125,7 @@ Image::apply_scaling(const Py::Tuple& args) {
   double sy = Py::Float(args[1]);
   
   //printf("applying scaling %1.2f, %1.2f\n", sx, sy);
-  agg::affine_matrix M = agg::scaling_matrix(sx, sy);	      
+  agg::trans_affine M = agg::trans_affine_scaling(sx, sy);	      
   srcMatrix *= M;
   imageMatrix *= M;
   
@@ -146,7 +149,7 @@ Image::apply_translation(const Py::Tuple& args) {
   double ty = Py::Float(args[1]);
   
   //printf("applying translation %1.2f, %1.2f\n", tx, ty);
-  agg::affine_matrix M = agg::translation_matrix(tx, ty);	      
+  agg::trans_affine M = agg::trans_affine_translation(tx, ty);	      
   srcMatrix *= M;
   imageMatrix *= M;
   
@@ -197,6 +200,40 @@ Image::as_str(const Py::Tuple& args) {
 }
 
 
+char Image::buffer_argb32__doc__[] = 
+"buffer = buffer_argb32)"
+"\n"
+"Return the image buffer as agbr32\n"
+;
+Py::Object 
+Image::buffer_argb32(const Py::Tuple& args) {
+  //"Return the image object as argb32";
+
+  _VERBOSE("RendererAgg::buffer_argb32");
+  
+  args.verify_length(0);    
+  int row_len = colsOut * 4;
+
+  unsigned char* buf_tmp = new unsigned char[row_len * rowsOut];
+  if (buf_tmp ==NULL) 
+    throw Py::MemoryError("RendererAgg::buffer_argb32 could not allocate memory");
+  
+  agg::rendering_buffer rtmp;
+  rtmp.attach(buf_tmp, colsOut, rowsOut, row_len);
+  
+  color_conv(&rtmp, rbufOut, agg::color_conv_rgba32_to_argb32());
+  
+  
+  //todo: how to do this with native CXX
+  //PyObject* o = Py_BuildValue("s#", buf_tmp, row_len * rowsOut);
+  PyObject* o = Py_BuildValue("lls#", rowsOut, colsOut, 
+			      buf_tmp, row_len * rowsOut);
+  delete [] buf_tmp;
+  return Py::asObject(o);
+
+
+}
+
 char Image::reset_matrix__doc__[] = 
 "reset_matrix()"
 "\n"
@@ -252,52 +289,114 @@ Image::resize(const Py::Tuple& args) {
   rb.clear(bg);
   agg::rasterizer_scanline_aa<> ras;
   agg::scanline_u8 sl;
-  
+
   
   //srcMatrix *= resizingMatrix;
   //imageMatrix *= resizingMatrix;
   imageMatrix.invert();
   interpolator_type interpolator(imageMatrix); 
   
-  // the image path
-  agg::path_storage path;
-  path.move_to(0, 0);
-  path.line_to(colsIn, 0);
-  path.line_to(colsIn, rowsIn);
-  path.line_to(0, rowsIn);
-  path.close_polygon();
-  agg::conv_transform<agg::path_storage> imageBox(path, srcMatrix);
-  ras.add_path(imageBox);
   
   
   
   agg::image_filter_base* filter = 0;  
   agg::span_allocator<agg::rgba8> sa;	
+  agg::rgba8 background(agg::rgba8(int(255*bg.r),
+				   int(255*bg.g),
+				   int(255*bg.b),
+				   int(255*bg.a)));
+
+
+
+
+  // the image path
+  agg::path_storage path;
+  agg::int8u *bufferPad = NULL;
+  agg::rendering_buffer rbufPad;
+
+  double x0, y0, x1, y1;
+  
+  if (interpolation==NEAREST) {
+    x0 = 0.0;
+    x1 = colsIn;
+    y0 = 0.0;
+    y1 = rowsIn;
+  }
+  else {
+    // if interpolation != nearest, create a new input buffer with the
+    // edges mirrored on all size.  Then new buffer size is colsIn+2 by
+    // rowsIn+2
+
+    x0 = 1.0;
+    x1 = colsIn+1;
+    y0 = 1.0;
+    y1 = rowsIn+1;
+
+    bufferPad = new agg::int8u[(rowsIn+2) * (colsIn+2) * BPP];
+    if (bufferPad ==NULL) 
+      throw Py::MemoryError("Image::resize could not allocate memory");
+    //pad the input buffer
+    for (size_t rowNum=0; rowNum<rowsIn+2; rowNum++)
+      for (size_t colNum=0; colNum<colsIn+2; colNum++) {
+	if ( (colNum==0) && (rowNum==1||(rowNum==rowsIn+1))) {
+	  //rewind to begining of column
+	  bufferIn -= colsIn * BPP; 
+	}
+	*bufferPad++ = *bufferIn++;       //red
+	*bufferPad++ = *bufferIn++;       //green
+	*bufferPad++ = *bufferIn++;       //blue
+	*bufferPad++ = *bufferIn++;       //alpha
+
+	//rewind one byte on the first and next to last columns
+	if ( colNum==0 || colNum==colsIn) bufferIn-=4;
+      }
+    //rewind the input buffers
+    bufferIn  -= rowsIn * colsIn * BPP;
+    bufferPad -= (rowsIn+2) * (colsIn+2) * BPP;
+    rbufPad.attach(bufferPad, colsIn+2, rowsIn+2, (colsIn+2) * BPP);
+  }
+
+
+
+    
+  if (buffer ==NULL) //todo: also handle allocation throw
+    throw Py::MemoryError("Image::resize could not allocate memory");
+  
+  path.move_to(x0, y0);
+  path.line_to(x1, y0);
+  path.line_to(x1, y1);
+  path.line_to(x0, y1);
+  path.close_polygon();
+  agg::conv_transform<agg::path_storage> imageBox(path, srcMatrix);
+  ras.add_path(imageBox);
+
   switch(interpolation)
     {
     case NEAREST:
       {
+	
 	typedef agg::span_image_filter_rgba32_nn<agg::order_rgba32,
 	  interpolator_type> span_gen_type;
-	typedef agg::renderer_scanline_u<renderer_base, span_gen_type> renderer_type;
-	
-	span_gen_type sg(sa, *rbufIn, agg::rgba(1,1,1,0), interpolator);
+	typedef agg::renderer_scanline_aa<renderer_base, span_gen_type> renderer_type;
+	       
+	span_gen_type sg(sa, *rbufIn, background, interpolator);
 	renderer_type ri(rb, sg);
-	ras.add_path(imageBox);
-	ras.render(sl, ri);
+	agg::render_scanlines(ras, sl, ri);
+	
       }
       break;
       
     case BILINEAR:
       {
+
+       
 	typedef agg::span_image_filter_rgba32_bilinear<agg::order_rgba32,
 	  interpolator_type> span_gen_type;
-	typedef agg::renderer_scanline_u<renderer_base, span_gen_type> renderer_type;
-	
-	span_gen_type sg(sa, *rbufIn, agg::rgba(1,1,1,0), interpolator);
+	typedef agg::renderer_scanline_aa<renderer_base, span_gen_type> renderer_type;
+		
+	span_gen_type sg(sa, rbufPad, background, interpolator);
 	renderer_type ri(rb, sg);
-	ras.add_path(imageBox);
-	ras.render(sl, ri);
+	agg::render_scanlines(ras, sl, ri);
       }
       break;
       
@@ -313,11 +412,10 @@ Image::resize(const Py::Tuple& args) {
       
       typedef agg::span_image_filter_rgba32<agg::order_rgba32,
 	interpolator_type> span_gen_type;
-      typedef agg::renderer_scanline_u<renderer_base, span_gen_type> renderer_type;
-      
-      span_gen_type sg(sa, *rbufIn, agg::rgba(1,1,1,0), interpolator, *filter);
+      typedef agg::renderer_scanline_aa<renderer_base, span_gen_type> renderer_type;
+      span_gen_type sg(sa, rbufPad, background, interpolator, *filter);
       renderer_type ri(rb, sg);
-      ras.render(sl, ri);
+      agg::render_scanlines(ras, sl, ri);
       
     }
   
@@ -329,8 +427,8 @@ Image::resize(const Py::Tuple& args) {
   
   
   bufferOut = buffer;
-  
-  
+  delete [] bufferPad;
+
   return Py::Object();
   
 }
@@ -441,26 +539,26 @@ Image::write_png(const Py::Tuple& args)
   
   fp = fopen(file_name, "wb");
   if (fp == NULL) 
-    throw Py::RuntimeError("could not open file");
+    throw Py::RuntimeError(Printf("Could not open file %s", file_name).str());
   
   
   png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
   if (png_ptr == NULL) {
     fclose(fp);
-    throw Py::RuntimeError("could not create write struct");
+    throw Py::RuntimeError("Could not create write struct");
   }
   
   info_ptr = png_create_info_struct(png_ptr);
   if (info_ptr == NULL) {
     fclose(fp);
     png_destroy_write_struct(&png_ptr, &info_ptr);
-    throw Py::RuntimeError("could not create info struct");
+    throw Py::RuntimeError("Could not create info struct");
   }
   
   if (setjmp(png_ptr->jmpbuf)) {
     fclose(fp);
     png_destroy_write_struct(&png_ptr, &info_ptr);
-    throw Py::RuntimeError("error building image");
+    throw Py::RuntimeError("Error building image");
   }
   
   png_init_io(png_ptr, fp);
@@ -519,6 +617,7 @@ Image::init_type() {
   add_varargs_method( "apply_scaling",	&Image::apply_scaling, Image::apply_scaling__doc__);
   add_varargs_method( "apply_translation", &Image::apply_translation, Image::apply_translation__doc__);
   add_varargs_method( "as_str", &Image::as_str, Image::as_str__doc__);
+  add_varargs_method( "buffer_argb32", &Image::buffer_argb32, Image::buffer_argb32__doc__);
   add_varargs_method( "get_aspect", &Image::get_aspect, Image::get_aspect__doc__);
   add_varargs_method( "get_interpolation", &Image::get_interpolation, Image::get_interpolation__doc__);
   add_varargs_method( "get_size", &Image::get_size, Image::get_size__doc__);
@@ -572,7 +671,7 @@ _image_module::from_images(const Py::Tuple& args) {
   
   imo->rbufOut = new agg::rendering_buffer;
   imo->rbufOut->attach(imo->bufferOut, imo->colsOut, imo->rowsOut, imo->colsOut * imo->BPP);
-  
+   
   pixfmt pixf(*imo->rbufOut);
   renderer_base rb(pixf);
   
@@ -627,7 +726,7 @@ _image_module::readpng(const Py::Tuple& args) {
 
   FILE *fp = fopen(fname.c_str(), "rb");
   if (!fp)
-    throw Py::RuntimeError("_image_module::readpng could not open PNG file for reading");
+    throw Py::RuntimeError(Printf("_image_module::readpng could not open PNG file %s for reading", fname.c_str()).str());
 
   fread(header, 1, 8, fp);
   if (png_sig_cmp(header, 0, 8))
@@ -671,8 +770,8 @@ _image_module::readpng(const Py::Tuple& args) {
   png_read_update_info(png_ptr, info_ptr);
   
   bool rgba = info_ptr->color_type == PNG_COLOR_TYPE_RGBA;
-  if ( (info_ptr->color_type != PNG_COLOR_TYPE_RGB) || rgba) {
-    std::cerr << "Found color type " << (int)info_ptr->color_type << std::endl;
+  if ( (info_ptr->color_type != PNG_COLOR_TYPE_RGB) && !rgba) {
+    std::cerr << "Found color type " << (int)info_ptr->color_type  << std::endl;
     throw Py::RuntimeError("_image_module::readpng: cannot handle color_type");
   }
   
@@ -737,7 +836,8 @@ _image_module::fromarray(const Py::Tuple& args) {
   
   Py::Object x = args[0];
   int isoutput = Py::Int(args[1]);
-  PyArrayObject *A = (PyArrayObject *) PyArray_ContiguousFromObject(x.ptr(), PyArray_DOUBLE, 2, 3); 
+  //PyArrayObject *A = (PyArrayObject *) PyArray_ContiguousFromObject(x.ptr(), PyArray_DOUBLE, 2, 3); 
+  PyArrayObject *A = (PyArrayObject *) PyArray_FromObject(x.ptr(), PyArray_DOUBLE, 2, 3); 
   
   if (A==NULL) 
     throw Py::ValueError("Array must be rank 2 or 3 of doubles"); 
@@ -748,31 +848,42 @@ _image_module::fromarray(const Py::Tuple& args) {
   imo->rowsIn  = A->dimensions[0];
   imo->colsIn  = A->dimensions[1];
   
-  
+
   size_t NUMBYTES(imo->colsIn * imo->rowsIn * imo->BPP);
   agg::int8u *buffer = new agg::int8u[NUMBYTES];  
   if (buffer==NULL) //todo: also handle allocation throw
     throw Py::MemoryError("_image_module::fromarray could not allocate memory");
-  
-  imo->bufferIn = buffer;
-  imo->rbufIn = new agg::rendering_buffer;
-  imo->rbufIn->attach(buffer, imo->colsIn, imo->rowsIn, imo->colsIn*imo->BPP);
-  
+
+  if (isoutput) {
+    // make the output buffer point to the input buffer
+    
+    imo->rowsOut  = imo->rowsIn;
+    imo->colsOut  = imo->colsIn;
+    
+    imo->rbufOut = new agg::rendering_buffer;
+    imo->bufferOut = buffer;
+    imo->rbufOut->attach(imo->bufferOut, imo->colsOut, imo->rowsOut, imo->colsOut * imo->BPP);
+    
+  }
+  else {
+    imo->bufferIn = buffer;
+    imo->rbufIn = new agg::rendering_buffer;
+    imo->rbufIn->attach(buffer, imo->colsIn, imo->rowsIn, imo->colsIn*imo->BPP);
+  }
   
   if   (A->nd == 2) { //assume luminance for now; 
     
     agg::int8u gray;
-    int start = 0;    
     for (size_t rownum=0; rownum<imo->rowsIn; rownum++) 
       for (size_t colnum=0; colnum<imo->colsIn; colnum++) {
 	
 	double val = *(double *)(A->data + rownum*A->strides[0] + colnum*A->strides[1]);
 	
 	gray = int(255 * val);
-	*(buffer+start++) = gray;       // red
-	*(buffer+start++) = gray;       // green
-	*(buffer+start++) = gray;       // blue
-	*(buffer+start++)   = 255;        // alpha
+	*buffer++ = gray;       // red
+	*buffer++ = gray;       // green
+	*buffer++ = gray;       // blue
+	*buffer++   = 255;        // alpha
       }
     
   }
@@ -780,16 +891,14 @@ _image_module::fromarray(const Py::Tuple& args) {
     
     if (A->dimensions[2] != 3 && A->dimensions[2] != 4 ) {
       Py_XDECREF(A);  
-      throw Py::ValueError("3rd dimension must be length 3 (RGB) or 4 (RGBA)"); 
+      throw Py::ValueError(Printf("3rd dimension must be length 3 (RGB) or 4 (RGBA); found %d", A->dimensions[2]).str()); 
       
     }
     
-    int rgba = A->dimensions[2]==4;
-    
-    int start = 0;    
+    int rgba = A->dimensions[2]==4;    
     double r,g,b,alpha;
     int offset =0;
-    
+
     for (size_t rownum=0; rownum<imo->rowsIn; rownum++) 
       for (size_t colnum=0; colnum<imo->colsIn; colnum++) {
 	offset = rownum*A->strides[0] + colnum*A->strides[1];
@@ -802,35 +911,21 @@ _image_module::fromarray(const Py::Tuple& args) {
 	else
 	  alpha = 1.0;
 	
-	*(buffer+start++) = int(255*r);         // red
-	*(buffer+start++) = int(255*g);         // green
-	*(buffer+start++) = int(255*b);         // blue
-	*(buffer+start++) = int(255*alpha);     // alpha
+	*buffer++ = int(255*r);         // red
+	*buffer++ = int(255*g);         // green
+	*buffer++ = int(255*b);         // blue
+	*buffer++ = int(255*alpha);     // alpha
 	
       }
+    
   } 
   else   { // error
     Py_XDECREF(A);  
     throw Py::ValueError("Illegal array rank; must be rank; must 2 or 3"); 
   }
+  buffer -= NUMBYTES;
   Py_XDECREF(A);  
   
-  if (isoutput) {
-    // make the output buffer point to the input buffer
-    
-    imo->rowsOut  = imo->rowsIn;
-    imo->colsOut  = imo->colsIn;
-    
-    imo->bufferOut = new agg::int8u[NUMBYTES];  
-    if (buffer == imo->bufferOut) //todo: also handle allocation throw
-      throw Py::MemoryError("_image_module::fromarray could not allocate memory");
-    
-    imo->rbufOut = new agg::rendering_buffer;
-    imo->rbufOut->attach(imo->bufferOut, imo->colsOut, imo->rowsOut, imo->colsOut * imo->BPP);
-    
-    for (size_t i=0; i<NUMBYTES; i++)
-      *(imo->bufferOut +i) = *(imo->bufferIn +i);
-  }
   return Py::asObject( imo );
 }
 
@@ -844,8 +939,14 @@ DL_EXPORT(void)
 #else
 void
 #endif
-init_image(void) {
-  _VERBOSE("init_image");
+
+#if defined(NUMARRAY)
+init_na_image(void) {
+  _VERBOSE("init_na_image");
+#else
+init_nc_image(void) {
+  _VERBOSE("init_nc_image");
+#endif
   
   static _image_module* _image = new _image_module;
   
